@@ -12,6 +12,6 @@ void action_unfocus_startup()
 static gboolean run_func(ObActionsData *data, gpointer options)
 {
     if (data->client && data->client == focus_client)
-        focus_fallback(FALSE, FALSE, TRUE);
+        focus_fallback(FALSE, FALSE, TRUE, FALSE);
     return FALSE;
 }
