@@ -32,7 +32,7 @@
 
 ObMenu *combined_menu;
 
-#define CLIENT -1
+#define SEPARATOR -1
 #define ADD_DESKTOP -2
 #define REMOVE_DESKTOP -3
 
@@ -49,7 +49,7 @@ static gboolean self_update(ObMenuFrame *frame, gpointer data)
         gboolean empty = TRUE;
         gboolean onlyiconic = TRUE;
 
-        menu_add_separator(menu, CLIENT, screen_desktop_names[desktop]);
+        menu_add_separator(menu, SEPARATOR, screen_desktop_names[desktop]);
         for (it = focus_order; it; it = g_list_next(it)) {
             ObClient *c = it->data;
             if (client_normal(c) && (!c->skip_taskbar || c->iconic) &&
@@ -61,11 +61,11 @@ static gboolean self_update(ObMenuFrame *frame, gpointer data)
 
                 if (c->iconic) {
                     gchar *title = g_strdup_printf("(%s)", c->icon_title);
-                    e = menu_add_normal(menu, CLIENT, title, NULL, FALSE);
+                    e = menu_add_normal(menu, desktop, title, NULL, FALSE);
                     g_free(title);
                 } else {
                     onlyiconic = FALSE;
-                    e = menu_add_normal(menu, CLIENT, c->title, NULL, FALSE);
+                    e = menu_add_normal(menu, desktop, c->title, NULL, FALSE);
                 }
 
                 if (config_menu_client_list_icons
@@ -87,7 +87,7 @@ static gboolean self_update(ObMenuFrame *frame, gpointer data)
             /* no entries or only iconified windows, so add a
              * way to go to this desktop without uniconifying a window */
             if (!empty)
-                menu_add_separator(menu, CLIENT, NULL);
+                menu_add_separator(menu, SEPARATOR, NULL);
 
             e = menu_add_normal(menu, desktop, _("Go there..."), NULL, TRUE);
             if (desktop == screen_desktop)
@@ -95,7 +95,7 @@ static gboolean self_update(ObMenuFrame *frame, gpointer data)
         }
     }
 
-    menu_add_separator(menu, CLIENT, _("Manage desktops"));
+    menu_add_separator(menu, SEPARATOR, _("Manage desktops"));
     menu_add_normal(menu, ADD_DESKTOP, _("_Add new desktop"), NULL, TRUE);
     menu_add_normal(menu, REMOVE_DESKTOP, _("_Remove last desktop"),
                     NULL, TRUE);
@@ -106,11 +106,7 @@ static gboolean self_update(ObMenuFrame *frame, gpointer data)
 static void menu_execute(ObMenuEntry *self, ObMenuFrame *f,
                          ObClient *c, guint state, gpointer data)
 {
-    if (self->id == CLIENT) {
-        if (self->data.normal.data) /* it's set to NULL if its destroyed */
-            client_activate(self->data.normal.data, FALSE, TRUE, TRUE, TRUE);
-    }
-    else if (self->id == ADD_DESKTOP) {
+    if (self->id == ADD_DESKTOP) {
         screen_add_desktop(FALSE);
         menu_frame_hide_all();
     }
@@ -118,8 +114,18 @@ static void menu_execute(ObMenuEntry *self, ObMenuFrame *f,
         screen_remove_desktop(FALSE);
         menu_frame_hide_all();
     }
-    else
-        screen_set_desktop(self->id, TRUE);
+    else {
+        ObClient *t = self->data.normal.data;
+        if (t) { /* it's set to NULL if its destroyed */
+            client_activate(t, FALSE, TRUE, TRUE, TRUE);
+            /* if the window is omnipresent then we need to go to its
+               desktop */
+            if (t->desktop == DESKTOP_ALL)
+                screen_set_desktop(self->id, FALSE);
+        }
+        else
+            screen_set_desktop(self->id, TRUE);
+    }
 }
 
 static void client_dest(ObClient *client, gpointer data)
