@@ -46,7 +46,8 @@ void focus_nothing();
 /*! Call this when you need to focus something! */
 struct _ObClient* focus_fallback(gboolean allow_refocus,
                                  gboolean allow_pointer,
-                                 gboolean allow_omnipresent);
+                                 gboolean allow_omnipresent,
+                                 gboolean focus_lost);
 
 /*! Add a new client into the focus order */
 void focus_order_add_new(struct _ObClient *c);
