@@ -26,7 +26,6 @@
 #include "screen.h"
 #include "openbox.h"
 #include "debug.h"
-#include "group.h"
 
 #include <X11/Xlib.h>
 #include <glib.h>
@@ -37,13 +36,10 @@ static gboolean focus_cycle_all_desktops;
 static gboolean focus_cycle_dock_windows;
 static gboolean focus_cycle_desktop_windows;
 
-static gboolean  focus_target_has_siblings  (ObClient *ft,
-                                             gboolean iconic_windows,
-                                             gboolean all_desktops);
-static ObClient *focus_find_directional    (ObClient *c,
-                                            ObDirection dir,
-                                            gboolean dock_windows,
-                                            gboolean desktop_windows);
+static ObClient *focus_find_directional(ObClient *c,
+                                        ObDirection dir,
+                                        gboolean dock_windows,
+                                        gboolean desktop_windows);
 
 void focus_cycle_startup(gboolean reconfig)
 {
@@ -60,115 +56,27 @@ void focus_cycle_stop(ObClient *ifclient)
     /* stop focus cycling if the given client is a valid focus target,
        and so the cycling is being disrupted */
     if (focus_cycle_target && ifclient &&
-        focus_cycle_target_valid(ifclient,
-                                 focus_cycle_iconic_windows,
-                                 focus_cycle_all_desktops,
-                                 focus_cycle_dock_windows,
-                                 focus_cycle_desktop_windows))
+        focus_valid_target(ifclient, TRUE,
+                           focus_cycle_iconic_windows,
+                           focus_cycle_all_desktops,
+                           focus_cycle_dock_windows,
+                           focus_cycle_desktop_windows))
     {
         focus_cycle(TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE);
         focus_directional_cycle(0, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE);
     }
 }
 
-/*! Returns if a focus target has valid group siblings that can be cycled
-  to in its place */
-static gboolean focus_target_has_siblings(ObClient *ft,
-                                          gboolean iconic_windows,
-                                          gboolean all_desktops)
-                                                         
-{
-    GSList *it;
-
-    if (!ft->group) return FALSE;
-
-    for (it = ft->group->members; it; it = g_slist_next(it)) {
-        ObClient *c = it->data;
-        /* check that it's not a helper window to avoid infinite recursion */
-        if (c != ft && c->type == OB_CLIENT_TYPE_NORMAL &&
-            focus_cycle_target_valid(c, iconic_windows, all_desktops, FALSE,
-                                     FALSE))
-        {
-            return TRUE;
-        }
-    }
-    return FALSE;
-}
-
-gboolean focus_cycle_target_valid(ObClient *ft,
-                                  gboolean iconic_windows,
-                                  gboolean all_desktops,
-                                  gboolean dock_windows,
-                                  gboolean desktop_windows)
-{
-    gboolean ok = FALSE;
-
-    /* it's on this desktop unless you want all desktops.
-
-       do this check first because it will usually filter out the most
-       windows */
-    ok = (all_desktops || ft->desktop == screen_desktop ||
-          ft->desktop == DESKTOP_ALL);
-
-    /* the window can receive focus somehow */
-    ok = ok && (ft->can_focus || ft->focus_notify);
-
-    /* the window is not iconic, or we're allowed to go to iconic ones */
-    ok = ok && (iconic_windows || !ft->iconic);
-
-    /* it's the right type of window */
-    if (dock_windows || desktop_windows)
-        ok = ok && ((dock_windows && ft->type == OB_CLIENT_TYPE_DOCK) ||
-                    (desktop_windows && ft->type == OB_CLIENT_TYPE_DESKTOP));
-    /* modal windows are important and can always get focus if they are
-       visible and stuff, so don't change 'ok' based on their type */ 
-    else if (!ft->modal)
-        /* normal non-helper windows are valid targets */
-        ok = ok &&
-            ((client_normal(ft) && !client_helper(ft))
-             ||
-             /* helper windows are valid targets if... */
-             (client_helper(ft) &&
-              /* ...a window in its group already has focus ... */
-              ((focus_client && ft->group == focus_client->group) ||
-               /* ... or if there are no other windows in its group 
-                  that can be cycled to instead */
-               !focus_target_has_siblings(ft, iconic_windows, all_desktops))));
-
-    /* it's not set to skip the taskbar (unless it is a type that would be
-       expected to set this hint, or modal) */
-    ok = ok && ((ft->type == OB_CLIENT_TYPE_DOCK ||
-                 ft->type == OB_CLIENT_TYPE_DESKTOP ||
-                 ft->type == OB_CLIENT_TYPE_TOOLBAR ||
-                 ft->type == OB_CLIENT_TYPE_MENU ||
-                 ft->type == OB_CLIENT_TYPE_UTILITY) ||
-                ft->modal ||
-                !ft->skip_taskbar);
-
-    /* it's not going to just send focus off somewhere else (modal window),
-       unless that modal window is not one of our valid targets, then let
-       you choose this window and bring the modal one here */
-    {
-        ObClient *cft = client_focus_target(ft);
-        ok = ok && (ft == cft || !focus_cycle_target_valid(cft,
-                                                           iconic_windows,
-                                                           all_desktops,
-                                                           dock_windows,
-                                                           desktop_windows));
-    }
-
-    return ok;
-}
-
-void focus_cycle(gboolean forward, gboolean all_desktops,
-                 gboolean dock_windows, gboolean desktop_windows,
-                 gboolean linear, gboolean interactive,
-                 gboolean dialog, gboolean done, gboolean cancel)
+ObClient* focus_cycle(gboolean forward, gboolean all_desktops,
+                      gboolean dock_windows, gboolean desktop_windows,
+                      gboolean linear, gboolean interactive,
+                      gboolean dialog, gboolean done, gboolean cancel)
 {
     static ObClient *t = NULL;
     static GList *order = NULL;
     GList *it, *start, *list;
     ObClient *ft = NULL;
+    ObClient *ret = NULL;
 
     if (interactive) {
         if (cancel) {
@@ -211,11 +119,11 @@ void focus_cycle(gboolean forward, gboolean all_desktops,
             if (it == NULL) it = g_list_last(list);
         }
         ft = it->data;
-        if (focus_cycle_target_valid(ft,
-                                     focus_cycle_iconic_windows,
-                                     focus_cycle_all_desktops,
-                                     focus_cycle_dock_windows,
-                                     focus_cycle_desktop_windows))
+        if (focus_valid_target(ft, TRUE,
+                               focus_cycle_iconic_windows,
+                               focus_cycle_all_desktops,
+                               focus_cycle_dock_windows,
+                               focus_cycle_desktop_windows))
         {
             if (interactive) {
                 if (ft != focus_cycle_target) { /* prevents flicker */
@@ -229,7 +137,7 @@ void focus_cycle(gboolean forward, gboolean all_desktops,
                                            focus_cycle_all_desktops,
                                            focus_cycle_dock_windows,
                                            focus_cycle_desktop_windows);
-                return;
+                return focus_cycle_target;
             } else if (ft != focus_cycle_target) {
                 focus_cycle_target = ft;
                 done = TRUE;
@@ -239,8 +147,7 @@ void focus_cycle(gboolean forward, gboolean all_desktops,
     } while (it != start);
 
 done_cycle:
-    if (done && focus_cycle_target)
-        client_activate(focus_cycle_target, FALSE, TRUE);
+    if (done && !cancel) ret = focus_cycle_target;
 
     t = NULL;
     focus_cycle_target = NULL;
@@ -252,13 +159,13 @@ done_cycle:
         focus_cycle_popup_hide();
     }
 
-    return;
+    return ret;
 }
 
 /* this be mostly ripped from fvwm */
 static ObClient *focus_find_directional(ObClient *c, ObDirection dir,
                                         gboolean dock_windows,
-                                        gboolean desktop_windows) 
+                                        gboolean desktop_windows)
 {
     gint my_cx, my_cy, his_cx, his_cy;
     gint offset = 0;
@@ -283,8 +190,8 @@ static ObClient *focus_find_directional(ObClient *c, ObDirection dir,
         /* the currently selected window isn't interesting */
         if (cur == c)
             continue;
-        if (!focus_cycle_target_valid(it->data, FALSE, FALSE, dock_windows,
-                                      desktop_windows))
+        if (!focus_valid_target(it->data, TRUE, FALSE, FALSE, dock_windows,
+                                desktop_windows))
             continue;
 
         /* find the centre coords of this window, from the
@@ -351,12 +258,15 @@ static ObClient *focus_find_directional(ObClient *c, ObDirection dir,
     return best_client;
 }
 
-void focus_directional_cycle(ObDirection dir, gboolean dock_windows,
-                             gboolean desktop_windows, gboolean interactive,
-                             gboolean dialog, gboolean done, gboolean cancel)
+ObClient* focus_directional_cycle(ObDirection dir, gboolean dock_windows,
+                                  gboolean desktop_windows,
+                                  gboolean interactive,
+                                  gboolean dialog,
+                                  gboolean done, gboolean cancel)
 {
     static ObClient *first = NULL;
     ObClient *ft = NULL;
+    ObClient *ret = NULL;
 
     if (cancel) {
         focus_cycle_target = NULL;
@@ -385,14 +295,14 @@ void focus_directional_cycle(ObDirection dir, gboolean dock_windows,
         GList *it;
 
         for (it = focus_order; it; it = g_list_next(it))
-            if (focus_cycle_target_valid(it->data,
-                                         focus_cycle_iconic_windows,
-                                         focus_cycle_all_desktops,
-                                         focus_cycle_dock_windows,
-                                         focus_cycle_desktop_windows))
+            if (focus_valid_target(it->data, TRUE,
+                                   focus_cycle_iconic_windows,
+                                   focus_cycle_all_desktops,
+                                   focus_cycle_dock_windows,
+                                   focus_cycle_desktop_windows))
                 ft = it->data;
     }
-        
+
     if (ft && ft != focus_cycle_target) {/* prevents flicker */
         focus_cycle_target = ft;
         if (!interactive)
@@ -406,11 +316,10 @@ void focus_directional_cycle(ObDirection dir, gboolean dock_windows,
                                       focus_cycle_all_desktops,
                                       focus_cycle_dock_windows,
                                       focus_cycle_desktop_windows);
-    return;
+    return focus_cycle_target;
 
 done_cycle:
-    if (done && focus_cycle_target)
-        client_activate(focus_cycle_target, FALSE, TRUE);
+    if (done && !cancel) ret = focus_cycle_target;
 
     first = NULL;
     focus_cycle_target = NULL;
@@ -418,5 +327,5 @@ done_cycle:
     focus_cycle_draw_indicator(NULL);
     focus_cycle_popup_single_hide();
 
-    return;
+    return ret;
 }
