@@ -22,8 +22,10 @@
 #include "openbox.h"
 #include "mainloop.h"
 #include "stacking.h"
+#include "grab.h"
 #include "client.h"
 #include "config.h"
+#include "actions.h"
 #include "screen.h"
 #include "menuframe.h"
 #include "keyboard.h"
@@ -57,7 +59,8 @@ static void parse_menu_separator(ObParseInst *i,
 static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
                        gpointer data);
 static gunichar parse_shortcut(const gchar *label, gboolean allow_shortcut,
-                               gchar **strippedlabel, guint *position);
+                               gchar **strippedlabel, guint *position,
+                               gboolean *always_show);
 
 
 static void client_dest(ObClient *client, gpointer data)
@@ -108,7 +111,7 @@ void menu_startup(gboolean reconfig)
             g_message(_("Unable to find a valid menu file '%s'"),
                       "menu.xml");
     }
-    
+
     g_assert(menu_parse_state.parent == NULL);
 
     if (!reconfig)
@@ -195,18 +198,20 @@ static ObMenu* menu_from_name(gchar *name)
         g_message(_("Attempted to access menu '%s' but it does not exist"),
                   name);
     return self;
-}  
+}
 
 #define VALID_SHORTCUT(c) (((c) >= '0' && (c) <= '9') || \
                            ((c) >= 'A' && (c) <= 'Z') || \
                            ((c) >= 'a' && (c) <= 'z'))
 
 static gunichar parse_shortcut(const gchar *label, gboolean allow_shortcut,
-                               gchar **strippedlabel, guint *position)
+                               gchar **strippedlabel, guint *position,
+                               gboolean *always_show)
 {
     gunichar shortcut = 0;
-    
+
     *position = 0;
+    *always_show = FALSE;
 
     g_assert(strippedlabel != NULL);
 
@@ -217,27 +222,34 @@ static gunichar parse_shortcut(const gchar *label, gboolean allow_shortcut,
 
         *strippedlabel = g_strdup(label);
 
-        /* if allow_shortcut is false, then you can't use the &, instead you
+        /* if allow_shortcut is false, then you can't use the '_', instead you
            have to just use the first valid character
         */
 
-        i = strchr(*strippedlabel, '&');
+        i = strchr(*strippedlabel, '_');
         if (allow_shortcut && i != NULL) {
-            /* there is an ampersand in the string */
+            /* there is an underscore in the string */
 
             /* you have to use a printable ascii character for shortcuts
-               don't allow space either, so you can have like "a & b"
+               don't allow space either, so you can have like "a _ b"
             */
             if (VALID_SHORTCUT(*(i+1))) {
                 shortcut = g_unichar_tolower(g_utf8_get_char(i+1));
                 *position = i - *strippedlabel;
+                *always_show = TRUE;
 
-                /* remove the & from the string */
+                /* remove the '_' from the string */
                 for (; *i != '\0'; ++i)
                     *i = *(i+1);
+            } else if (*(i+1) == '\0') {
+                /* no default shortcut if the '_' is the last character
+                   (eg. "Exit_") for menu entries that you don't want
+                   to be executed by mistake
+                */
+                    *i = '\0';
             }
         } else {
-            /* there is no ampersand, so find the first valid character to use
+            /* there is no underscore, so find the first valid character to use
                instead */
 
             for (i = *strippedlabel; *i != '\0'; ++i)
@@ -256,19 +268,18 @@ static void parse_menu_item(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
 {
     ObMenuParseState *state = data;
     gchar *label;
-    
+
     if (state->parent) {
         if (parse_attr_string("label", node, &label)) {
             GSList *acts = NULL;
 
             for (node = node->children; node; node = node->next)
                 if (!xmlStrcasecmp(node->name, (const xmlChar*) "action")) {
-                    ObAction *a = action_parse
-                        (i, doc, node, OB_USER_ACTION_MENU_SELECTION);
+                    ObActionsAct *a = actions_parse(i, doc, node);
                     if (a)
                         acts = g_slist_append(acts, a);
                 }
-            menu_add_normal(state->parent, -1, label, acts, FALSE);
+            menu_add_normal(state->parent, -1, label, acts, TRUE);
             g_free(label);
         }
     }
@@ -305,7 +316,7 @@ static void parse_menu(ObParseInst *i, xmlDocPtr doc, xmlNodePtr node,
         if (!parse_attr_string("label", node, &title))
             goto parse_menu_fail;
 
-        if ((menu = menu_new(name, title, FALSE, NULL))) {
+        if ((menu = menu_new(name, title, TRUE, NULL))) {
             menu->pipe_creator = state->pipe_creator;
             if (parse_attr_string("execute", node, &script)) {
                 menu->execute = parse_expand_tilde(script);
@@ -339,7 +350,8 @@ ObMenu* menu_new(const gchar *name, const gchar *title,
     self->data = data;
 
     self->shortcut = parse_shortcut(title, allow_shortcut_selection,
-                                    &self->title, &self->shortcut_position);
+                                    &self->title, &self->shortcut_position,
+                                    &self->shortcut_always_show);
 
     g_hash_table_replace(menu_hash, self->name, self);
 
@@ -406,13 +418,13 @@ static gboolean menu_hide_delay_func(gpointer data)
     return FALSE; /* no repeat */
 }
 
-void menu_show(gchar *name, gint x, gint y, gint button, ObClient *client)
+void menu_show(gchar *name, gint x, gint y, gboolean mouse, ObClient *client)
 {
     ObMenu *self;
     ObMenuFrame *frame;
 
-    if (!(self = menu_from_name(name))
-        || keyboard_interactively_grabbed()) return;
+    if (!(self = menu_from_name(name)) ||
+        grab_on_keyboard() || grab_on_pointer()) return;
 
     /* if the requested menu is already the top visible menu, then don't
        bother */
@@ -428,32 +440,35 @@ void menu_show(gchar *name, gint x, gint y, gint button, ObClient *client)
     menu_clear_pipe_caches();
 
     frame = menu_frame_new(self, 0, client);
-    if (!menu_frame_show_topmenu(frame, x, y, button))
+    if (!menu_frame_show_topmenu(frame, x, y, mouse))
         menu_frame_free(frame);
-    else if (!button) {
-        /* select the first entry if it's not a submenu and we opened
-         * the menu with the keyboard, and skip all headers */
-        GList *it = frame->entries;
-        while (it) {
-            ObMenuEntryFrame *e = it->data;
-            if (e->entry->type == OB_MENU_ENTRY_TYPE_NORMAL) {
-                menu_frame_select(frame, e, FALSE);
-                break;
-            } else if (e->entry->type == OB_MENU_ENTRY_TYPE_SEPARATOR)
-                it = g_list_next(it);
-            else
-                break;
-        }
-    }
-
-    if (!button)
-        menu_can_hide = TRUE;
     else {
-        menu_can_hide = FALSE;
-        ob_main_loop_timeout_add(ob_main_loop,
-                                 config_menu_hide_delay * 1000,
-                                 menu_hide_delay_func,
-                                 NULL, g_direct_equal, NULL);
+        if (!mouse) {
+            /* select the first entry if it's not a submenu and we opened
+             * the menu with the keyboard, and skip all headers */
+            GList *it = frame->entries;
+            while (it) {
+                ObMenuEntryFrame *e = it->data;
+                if (e->entry->type == OB_MENU_ENTRY_TYPE_NORMAL) {
+                    menu_frame_select(frame, e, FALSE);
+                    break;
+                } else if (e->entry->type == OB_MENU_ENTRY_TYPE_SEPARATOR)
+                    it = g_list_next(it);
+                else
+                    break;
+            }
+        }
+
+        /* reset the hide timer */
+        if (!mouse)
+            menu_can_hide = TRUE;
+        else {
+            menu_can_hide = FALSE;
+            ob_main_loop_timeout_add(ob_main_loop,
+                                     config_menu_hide_delay * 1000,
+                                     menu_hide_delay_func,
+                                     NULL, g_direct_equal, NULL);
+        }
     }
 }
 
@@ -498,7 +513,7 @@ void menu_entry_unref(ObMenuEntry *self)
         case OB_MENU_ENTRY_TYPE_NORMAL:
             g_free(self->data.normal.label);
             while (self->data.normal.actions) {
-                action_unref(self->data.normal.actions->data);
+                actions_act_unref(self->data.normal.actions->data);
                 self->data.normal.actions =
                     g_slist_delete_link(self->data.normal.actions,
                                         self->data.normal.actions);
@@ -670,7 +685,8 @@ void menu_entry_set_label(ObMenuEntry *self, const gchar *label,
         g_free(self->data.normal.label);
         self->data.normal.shortcut =
             parse_shortcut(label, allow_shortcut, &self->data.normal.label,
-                           &self->data.normal.shortcut_position);
+                           &self->data.normal.shortcut_position,
+                           &self->data.normal.shortcut_always_show);
         break;
     default:
         g_assert_not_reached();
