@@ -35,41 +35,77 @@
 
 static void set_modkey_mask(guchar mask, KeySym sym);
 
-static XModifierKeymap *modmap;
+/* This is 8 lists of keycodes that are bound to the given mod mask.
+   If contains more than the one given to us by X cuz XKB is weird apparently.
+   We will look up all keycodes for a given keysym that is bound to the mask,
+   and add them all here.
+
+   With XKB, you can have a keycode bound to a modifier that isn't in the
+   modifier map somehow.  So this means that when we try translate from the
+   KeyRelease to a mod mask, we are unable to.  So this array stores *all*
+   the KeyCodes for each KeySym for each KeyCode bound to a mod mask.
+   Confused? Haha...
+
+   ModMask -> n KeyCodes -> n*m KeySyms (up to m for each KeyCode) ->
+     n*m*p KeyCodes (up to p for each KeySym)
+*/
+static GArray *modmap[NUM_MASKS];
 static KeySym *keymap;
 static gint min_keycode, max_keycode, keysyms_per_keycode;
 /* This is a bitmask of the different masks for each modifier key */
 static guchar modkeys_keys[OB_MODKEY_NUM_KEYS];
 
+static gboolean alt_l = FALSE;
+static gboolean meta_l = FALSE;
+static gboolean super_l = FALSE;
+static gboolean hyper_l = FALSE;
+
 void modkeys_startup(gboolean reconfigure)
 {
-    gint i, j, k;
+    static XModifierKeymap *xmodmap;
+    gint i, j, k, l, m;
 
     /* reset the keys to not be bound to any masks */
     for (i = 0; i < OB_MODKEY_NUM_KEYS; ++i)
         modkeys_keys[i] = 0;
 
-    modmap = XGetModifierMapping(ob_display);
-    g_assert(modmap->max_keypermod > 0);
+    xmodmap = XGetModifierMapping(ob_display);
+    g_assert(xmodmap->max_keypermod > 0);
 
     XDisplayKeycodes(ob_display, &min_keycode, &max_keycode);
     keymap = XGetKeyboardMapping(ob_display, min_keycode,
                                  max_keycode - min_keycode + 1,
                                  &keysyms_per_keycode);
 
+    alt_l = meta_l = super_l = hyper_l = FALSE;
+
     /* go through each of the modifier masks (eg ShiftMask, CapsMask...) */
     for (i = 0; i < NUM_MASKS; ++i) {
+        /* reset the modmap list */
+        modmap[i] = g_array_new(FALSE, FALSE, sizeof(KeyCode));
+
         /* go through each keycode that is bound to the mask */
-        for (j = 0; j < modmap->max_keypermod; ++j) {
+        for (j = 0; j < xmodmap->max_keypermod; ++j) {
             KeySym sym;
             /* get a keycode that is bound to the mask (i) */
-            KeyCode keycode = modmap->modifiermap[i*modmap->max_keypermod + j];
+            KeyCode keycode = xmodmap->modifiermap[i*xmodmap->max_keypermod+j];
             if (keycode) {
                 /* go through each keysym bound to the given keycode */
                 for (k = 0; k < keysyms_per_keycode; ++k) {
                     sym = keymap[(keycode-min_keycode) * keysyms_per_keycode +
                                  k];
                     if (sym != NoSymbol) {
+                        /* find all keycodes for the given keysym */
+                        for (l = min_keycode; l <= max_keycode; ++l)
+                            for (m = 0; m < keysyms_per_keycode; ++m)
+                                if (keymap[(l-min_keycode) *
+                                           keysyms_per_keycode + m] == sym)
+                                {
+                                    /* add all keycodes for the keysym to our
+                                       modmap */
+                                    g_array_append_val(modmap[i], l);
+                                }
+
                         /* bind the key to the mask (e.g. Alt_L => Mod1Mask) */
                         set_modkey_mask(nth_mask(i), sym);
                     }
@@ -82,17 +118,22 @@ void modkeys_startup(gboolean reconfigure)
     modkeys_keys[OB_MODKEY_KEY_CAPSLOCK] = LockMask;
     modkeys_keys[OB_MODKEY_KEY_SHIFT] = ShiftMask;
     modkeys_keys[OB_MODKEY_KEY_CONTROL] = ControlMask;
+
+    XFreeModifiermap(xmodmap);
 }
 
 void modkeys_shutdown(gboolean reconfigure)
 {
-    XFreeModifiermap(modmap);
+    guint i;
+
+    for (i = 0; i < NUM_MASKS; ++i)
+        g_array_free(modmap[i], TRUE);
     XFree(keymap);
 }
 
 guint modkeys_keycode_to_mask(guint keycode)
 {
-    gint i, j;
+    guint i, j;
     guint mask = 0;
 
     if (keycode == NoSymbol) return 0;
@@ -100,9 +141,9 @@ guint modkeys_keycode_to_mask(guint keycode)
     /* go through each of the modifier masks (eg ShiftMask, CapsMask...) */
     for (i = 0; i < NUM_MASKS; ++i) {
         /* go through each keycode that is bound to the mask */
-        for (j = 0; j < modmap->max_keypermod; ++j) {
+        for (j = 0; j < modmap[i]->len; ++j) {
             /* compare with a keycode that is bound to the mask (i) */
-            if (modmap->modifiermap[i*modmap->max_keypermod + j] == keycode)
+            if (g_array_index(modmap[i], KeyCode, j) == keycode)
                 mask |= nth_mask(i);
         }
     }
@@ -135,14 +176,37 @@ static void set_modkey_mask(guchar mask, KeySym sym)
         modkeys_keys[OB_MODKEY_KEY_NUMLOCK] |= mask;
     else if (sym == XK_Scroll_Lock)
         modkeys_keys[OB_MODKEY_KEY_SCROLLLOCK] |= mask;
-    else if (sym == XK_Super_L || sym == XK_Super_R)
+
+    else if (sym == XK_Super_L && super_l)
         modkeys_keys[OB_MODKEY_KEY_SUPER] |= mask;
-    else if (sym == XK_Hyper_L || sym == XK_Hyper_R)
+    else if (sym == XK_Super_L && !super_l)
+        /* left takes precident over right, so erase any masks the right
+           key may have set */
+        modkeys_keys[OB_MODKEY_KEY_SUPER] = mask, super_l = TRUE;
+    else if (sym == XK_Super_R && !super_l)
+        modkeys_keys[OB_MODKEY_KEY_SUPER] |= mask;
+
+    else if (sym == XK_Hyper_L && hyper_l)
         modkeys_keys[OB_MODKEY_KEY_HYPER] |= mask;
-    else if (sym == XK_Alt_L || sym == XK_Alt_R)
+    else if (sym == XK_Hyper_L && !hyper_l)
+        modkeys_keys[OB_MODKEY_KEY_HYPER] = mask, hyper_l = TRUE;
+    else if (sym == XK_Hyper_R && !hyper_l)
+        modkeys_keys[OB_MODKEY_KEY_HYPER] |= mask;
+
+    else if (sym == XK_Alt_L && alt_l)
         modkeys_keys[OB_MODKEY_KEY_ALT] |= mask;
-    else if (sym == XK_Meta_L || sym == XK_Meta_R)
+    else if (sym == XK_Alt_L && !alt_l)
+        modkeys_keys[OB_MODKEY_KEY_ALT] = mask, alt_l = TRUE;
+    else if (sym == XK_Alt_R && !alt_l)
+        modkeys_keys[OB_MODKEY_KEY_ALT] |= mask;
+
+    else if (sym == XK_Meta_L && meta_l)
         modkeys_keys[OB_MODKEY_KEY_META] |= mask;
+    else if (sym == XK_Meta_L && !meta_l)
+        modkeys_keys[OB_MODKEY_KEY_META] = mask, meta_l = TRUE;
+    else if (sym == XK_Meta_R && !meta_l)
+        modkeys_keys[OB_MODKEY_KEY_META] |= mask;
+
     /* CapsLock, Shift, and Control are special and hard-coded */
 }
 
