@@ -132,7 +132,7 @@ typedef struct Atoms {
 /*  Atom net_wm_pid; */
     Atom net_wm_allowed_actions;
     Atom net_wm_user_time;
-    Atom net_wm_user_time_window;
+/*  Atom net_wm_user_time_window; */
     Atom net_frame_extents;
 
     /* application protocols */
