@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: nil; tab-width: 4; c-basic-offset: 4; -*-
 
    session.h for the Openbox window manager
-   Copyright (c) 2003        Ben Jansens
+   Copyright (c) 2003-2007   Dana Jansens
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,30 +19,38 @@
 #ifndef __ob__session_h
 #define __ob__session_h
 
-#include <glib.h>
+#include "client.h"
+#include "screen.h"
 
-struct _ObClient;
+#include <glib.h>
 
 typedef struct _ObSessionState ObSessionState;
 
 struct _ObSessionState {
-    gchar *id, *name, *class, *role;
-    guint stacking;
+    gchar *id, *command, *name, *class, *role;
+    ObClientType type;
     guint desktop;
     gint x, y, w, h;
     gboolean shaded, iconic, skip_pager, skip_taskbar, fullscreen;
-    gboolean above, below, max_horz, max_vert;
+    gboolean above, below, max_horz, max_vert, undecorated;
+    gboolean focused;
 
     gboolean matched;
 };
 
+/*! The desktop being viewed when the session was saved. A valud of -1 means
+  it was not saved */
+extern gint session_desktop;
+extern gint session_num_desktops;
+extern gboolean session_desktop_layout_present;
+extern ObDesktopLayout session_desktop_layout;
+extern GSList *session_desktop_names;
+
 extern GList *session_saved_state;
 
-void session_startup(gint *argc, gchar ***argv);
-void session_shutdown();
+void session_startup(gint argc, gchar **argv);
+void session_shutdown(gboolean permanent);
 
 GList* session_state_find(struct _ObClient *c);
-gboolean session_state_cmp(ObSessionState *s, struct _ObClient *c);
-void session_state_free(ObSessionState *state);
 
 #endif
